# We should add a functionality to subset on the samples to be prepared for really large cohorts, question is how specific to the tally file format this can be
# 'names', 'dims' 'range' need to come after '...' since they can be missing.
h5readBlock <- function( filename, group, names, dims, range, samples = NULL, sampleDimMap = .sampleDimMap, verbose = FALSE ){
  sampleData <- getSampleData( filename, group )
  if( !is.null(samples) ){
    stopifnot(all(samples %in% sampleData$Sample))
  }else{
    samples = sampleData$Sample  
  }
  selectedColumns <- subset( sampleData, Sample %in% samples )$Column
  itemlist = h5ls( filename )
  itemlist = itemlist[grep( group, itemlist$group ), ]
  datasets = itemlist[ itemlist$group == group & itemlist$otype == "H5I_DATASET", ]
  if(nrow(datasets)==0)
    stop(sprintf("No datasets match the specified value of 'group': %s.", paste(group, collapse=", ")))
  
  if(!missing(names)) {
    if(!all(names %in% datasets$name))
      warning("Not all datasets specified in the 'names' argument are present in the file, your function calls might fail!")
    datasets = datasets[ datasets$name %in% names, ]
    name_order = sapply( names, function(name) which( datasets$name == name ) ) ## to do: use 'match'
    datasets = datasets[ name_order, ]
  }
  datasets$DimCount = sapply( strsplit( datasets$dim, split=" x " ), length )
  f <- H5Fopen(filename, flags = "H5F_ACC_RDONLY")
  g <- H5Gopen(f, group )
  if( missing(dims) ){
    if( !("Counts" %in% itemlist$name) ){
      stop( "If dims is not specified the dataset 'Counts' must exist in the tallyFile" )
    }
    d = H5Dopen(g, "Counts")
    s = H5Dget_space(d)
    dimensions = as.list(H5Sget_simple_extent_dims(s)$size)
    base::names( dimensions ) = c( "Base", "Sample", "Strand", "Position" )
    datasets$PosDim = sapply( strsplit( datasets$dim, split=" x " ), function(x) which( x == dimensions$Position ) )
    H5Sclose(s)
    H5Dclose(d)
  }else{
    if(missing(names))
      stop("If argument 'dims' is specified, argument 'names' must be provided, too, to avoid ambiguous results.")
    datasets$PosDim = dims
    if( sum( dims <= datasets$DimCount & dims >= 1) < nrow(datasets) ) 
      stop( "The specified dimensions 'dims' do not match the structure of the data" )
  }
  
  chromlength <- sapply(
    seq(nrow(datasets)),
    function( i ){
      d <- H5Dopen(g, datasets$name[i])
      s <- H5Dget_space(d)
      ret <- H5Sget_simple_extent_dims(s)$size[datasets$PosDim[i]]
      H5Sclose(s)
      H5Dclose(d)
      return(ret)
    })
  if( length(unique(chromlength)) == 1 ){
    chromlength = chromlength[1]
  }else{
    stop( "The datasets in specified do not have a common length in the specified dimensions!" )
  }
  
  if( missing(range) ){
    range = c( 1, chromlength )
  }
  H5Gclose(g)
  H5Fclose(f)
  
  stopifnot( is.numeric(range), length(range) == 2, diff(range) > 0, range[1] >= 1, range[2] <= chromlength )
  blockstart = range[1]
  blockend   = range[2]
  if(verbose)
    message( "Reading Block ", blockstart, ": ", blockend, sep="")
  vec = vector(mode="list", length=nrow(datasets))
  names(vec) = datasets$name 
  for( i in seq_len(nrow(datasets)) ){
    idx_list = vector(mode="list", length=datasets$DimCount[i] )
    idx_list[[ datasets$PosDim[i] ]] = blockstart:blockend
    sampleDim <- sampleDimMap[[datasets$name[i]]]
    if(!is.null(sampleDim)){
      if( sampleDim == datasets$PosDim[i] & !all( sampleData$Sample %in% samples) ){
        stop("Specified binning dimension equals sample dimension - can't subset by sample")
      }
      idx_list[[ sampleDim ]] <- selectedColumns
    }
    #vec[[i]] = h5read( filename, paste( group, datasets$name[i], sep="/" ), index = idx_list, bit64conversion='bit64' )
    vec[[i]] = h5read( filename, paste( group, datasets$name[i], sep="/" ), index = idx_list )
  }
  vec$h5dapplyInfo <- list( Blockstart = blockstart, Blockend = blockend, Datasets = datasets, Group = group )
  return( vec )
}
